package de.ids_mannheim.korap.openthesaurus;

import java.io.*;
import java.util.*;
import java.util.regex.*;

public class ThesaurusDictionary {
    static String regex = "(\\(.+\\) *)*([^\\(\\)]+[^\\(\\ )])( *\\(.+\\))*";
    static Pattern withoutParenthesesPattern = Pattern.compile(regex);

    static String cleanString(String str) {
        Matcher withoutParentheses = withoutParenthesesPattern.matcher(str);
        if (withoutParentheses.matches()) {
            return withoutParentheses.group(2);
        }
        return str;
    }

    public static HashMap<String, List<List<String>>> createThesaurusDictionary(InputStream thesaurusPath) {

        //File thesaurusData = new File(thesaurusPath);
        HashMap<String, List<List<String>>> thesaurusDict = new HashMap<>();
        try (BufferedReader bf = new BufferedReader(new InputStreamReader(thesaurusPath))) {
            Iterator<String> fileItr = bf.lines().iterator();

            while (fileItr.hasNext()) {
                String line = fileItr.next();
                if (line.startsWith("==")) break;
                if (!line.startsWith("#") && line.length()>1) {
                    String[] lineSplit = line.split(";");
                    if (lineSplit.length > 1) {
                        // this creates a dictionary entry for every word on the line
                        // with all the other words as a List HashSet of synonyms
                        for (int i = 0; i < lineSplit.length; i++) {
                            String currentKey = lineSplit[i];
                            List<String> synonyms = new ArrayList<>();
                            for (int j = 0; j < lineSplit.length; j++) {
                                // include all words except the current key
                                // don't include explanations and word usage as "(ugs.)"
                                if (j != i) synonyms.add(cleanString(lineSplit[j]));
                            }
                            if (thesaurusDict.containsKey(currentKey)) {
                                thesaurusDict.get(currentKey).add(synonyms);
                            }
                            else {
                                List<List<String>> synsetList = new ArrayList<>();
                                synsetList.add(synonyms);
                                thesaurusDict.put(lineSplit[i], synsetList);
                            }
                        }
                    }
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return thesaurusDict;
    }

    public static HashMap<String, List<List<String>>> createThesaurusDictionary(String thesaurusPath) {

        InputStream FIS = null;

        try {
            FIS = new FileInputStream(thesaurusPath);
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        return createThesaurusDictionary(FIS);

    }
    /*
    public static void main(String[] args) {

        HashMap<String, HashSet<String>> thesaurusDict =
                createThesaurusDictionary("openthesaurus_data/openthesaurus.txt");

        System.out.println(thesaurusDict.get("laufen"));
    }
     */
}
